@@ -7,13 +7,20 @@
  */
 
 #include "c-config.h"
-#include "Curses.h"
+#include "pCurses.h"
 
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
 
+#ifdef __PDCURSES__
+#undef SP
+#endif
+
+#define C_XS_VERSION  1.02
+
 typedef WINDOW *Curses;
+typedef SCREEN *Screen;
 
 /*
  * Begin support variables and functions
@@ -79,6 +86,19 @@ int argnum;
 	croak("argument is not a Curses window");
 }
 
+static Screen
+c_sv2Screen(sv, argnum)
+SV *sv;
+int argnum;
+{
+    if (sv_isa(sv, "Curses::Screen")) { return (Screen)SvIV((SV*)SvRV(sv)); }
+    if (argnum >= 0)
+	croak("argument %d to Curses function '%s' is not a Curses screen",
+	      argnum, c_function);
+    else
+	croak("argument is not a Curses screen");
+}
+
 /* Fix cast to do the "right thing" for characters bigger than 128.
  * Shame on me for being ASCII-centric.  Thanks to
  * win@incom.rhein-main.de (Winfried Koenig).
@@ -2136,7 +2156,7 @@ XS(XS_Curses_savetty)
 {
     dXSARGS;
 #ifdef C_SAVETTY
-    c_exactargs("savetty");
+    c_exactargs("savetty", items, 0);
     {
 	savetty();
     }
@@ -2680,6 +2700,56 @@ XS(XS_Curses_vline)
     XSRETURN(1);
 }
 
+/* Screen stuff */
+/* Contributed by Santeri Paavolainen <sjpaavol@cc.helsinki.fi> */
+
+XS(XS_Curses_newterm)
+{
+    dXSARGS;
+#ifdef C_NEWTERM
+    c_exactargs("newterm", items, 3);
+    {
+	char *	term;
+	FILE *	out	= IoIFP(sv_2io(ST(1)));
+	FILE *	in	= IoIFP(sv_2io(ST(2)));
+	Screen	ret;
+
+	/* The 'term' parameter must be null terminated */
+	if (ST(0) != &sv_undef)
+	  term = (char *)SvPV(ST(0),na);
+	else
+	  term = NULL;
+
+	ret   = newterm(term, out, in);
+	ST(0) = sv_newmortal();
+	sv_setref_pv(ST(0), "Curses::Screen", (void *)ret);
+    }
+#else
+    c_not_there("newterm");
+#endif
+    XSRETURN(1);
+}
+
+XS(XS_Curses_set_term)
+{
+    dXSARGS;
+#ifdef	C_SET_TERM
+    c_countargs("set_term", items, 1);
+    {
+	Screen	scr = c_sv2Screen(ST(0), 0);
+	Screen	ret = set_term(scr);
+
+	ST(0) = sv_newmortal();
+	sv_setref_pv(ST(0), "Curses::Screen", (void *) ret);
+    }
+#else
+    c_not_there("set_term");
+#endif
+    XSRETURN(1);
+}
+
+/* Tied stuff */
+
 XS(XS_Curses_Vars_TIESCALAR)
 {
     dXSARGS;
@@ -2784,6 +2854,36 @@ XS(XS_Curses_curscr)
     XSRETURN(1);
 }
 
+XS(XS_Curses_LINES)
+{
+    dXSARGS;
+    c_exactargs("LINES", items, 0);
+    {
+        ST(0) = sv_newmortal();
+        sv_setiv(ST(0), (IV)LINES);
+    }
+    XSRETURN(1);
+}
+
+XS(XS_Curses_COLS)
+{
+    dXSARGS;
+    c_exactargs("COLS", items, 0);
+    {
+        ST(0) = sv_newmortal();
+        sv_setiv(ST(0), (IV)COLS);
+    }
+    XSRETURN(1);
+}
+
+XS(XS_Curses_XS_VERSION)
+{
+  dXSARGS;
+  ST(0) = sv_newmortal();
+  sv_setnv(ST(0), (double)C_XS_VERSION);
+  XSRETURN(1);
+}
+
 XS(XS_Curses_constant)
 {
     dXSARGS;
@@ -3362,9 +3462,16 @@ XS(boot_Curses)
     newXS("Curses::ungetch",		XS_Curses_ungetch,		file);
     newXS("Curses::vline",		XS_Curses_vline,		file);
 
+    newXS("Curses::newterm",		XS_Curses_newterm,		file);
+    newXS("Curses::set_term",		XS_Curses_set_term,		file);
+    
     newXS("Curses::constant",		XS_Curses_constant,		file);
     newXS("Curses::curscr",		XS_Curses_curscr,		file);
     newXS("Curses::stdscr",		XS_Curses_stdscr,		file);
+    newXS("Curses::LINES",		XS_Curses_LINES,		file);
+    newXS("Curses::COLS",		XS_Curses_COLS,			file);
+    newXS("Curses::XS_VERSION",		XS_Curses_XS_VERSION,		file);
+
     newXS("Curses::Vars::DESTROY",	XS_Curses_Vars_DESTROY,		file);
     newXS("Curses::Vars::FETCH",	XS_Curses_Vars_FETCH,		file);
     newXS("Curses::Vars::STORE",	XS_Curses_Vars_STORE,		file);
